
(* Copyright 1988 Wolfram Research Inc. *)

		(**  Graphics with Polyhedra  **)

(* S. Omohundro and S. Wolfram, October 1987; R. Maeder, February 1988 *)

Begin["Polyhedra`"]

(* Platonic solids *)

Icosahedron::usage =
	"Icosahedron[(pos:{0,0,0}, (scale:1))] gives a graphics 
	object representing an icosahedron with specified center 
	position and scale."

Dodecahedron::usage =
        "Dodecahedron[(pos:{0,0,0}, (scale:1))] gives a graphics
        object representing a dodecahedron with specified center
        position and scale."

Octahedron::usage =
        "Octahedron[(pos:{0,0,0}, (scale:1))] gives a graphics
        object representing an octahedron with specified center
        position and scale."

Cube::usage =
        "Cube[(pos:{0,0,0}, (scale:1))] gives a graphics
        object representing a cube with specified center
        position and scale."

Tetrahedron::usage =
        "Tetrahedron[(pos:{0,0,0}, (scale:1))] gives a graphics
        object representing a tetrahedron with specified center
        position and scale."

(* Stellations *)

Stellate::usage =
	"Stellate[polyhedron, (stellation ratio:2)] generates a stellated
	polyhedron from a polygon list. Negative stellation ratios give
	concave figures."



MakePolyhedron[coords_, pos:{_?NumberQ, _?NumberQ, _?NumberQ},
						scale_?NumberQ] :=
	Map[Polygon, Map[# + pos &, scale coords, {2}] ]

MakePolyhedron[coords_, pos_] := MakePolyhedron[coords, pos, 1.]

MakePolyhedron[coords_] := MakePolyhedron[coords, {0.,0.,0.}, 1.]


Icosahedron[opts___] := MakePolyhedron[`IcosahedronCoords, opts]

Dodecahedron[opts___] := MakePolyhedron[`DodecahedronCoords, opts] 

Octahedron[opts___] := MakePolyhedron[`OctahedronCoords, opts]

Cube[opts___] := MakePolyhedron[`CubeCoords, opts] 

Tetrahedron[opts___] := MakePolyhedron[`TetrahedronCoords, opts]


StellateFace[face_List, k_] :=
	Block[ { apex, 
		 n = Length[face] } ,
		apex = N [ k Apply[Plus, face] / n ] ;
		Table[ Polygon[ {apex, face[[i]], face[[ Mod[i, n] + 1 ]] }
			      ],
		     {i, n} ]
	]
	
Stellate[poly_, k_?NumberQ] := 
	Flatten [ poly /. Polygon[x_] :> StellateFace[x, k]  ]

Stellate[poly_] := Stellate[poly, 2.]


(** polygon specifications for unit polyhedra **)

`IcosahedronCoords =
{{{-1.89376, 0.108633, 0.141015}, {-0.789696, 1.67778, -0.423627}, 
      {-0.970542, 0.0952288, -1.6331}}, 
     {{-0.789696, 1.67778, -0.423627}, {-1.89376, 0.108633, 0.141015}, 
      {-0.68793, 1.00884, 1.45843}}, 
     {{-1.89376, 0.108633, 0.141015}, {-0.970542, 0.0952288, -1.6331}, 
      {-0.980538, -1.55178, -0.498532}}, 
     {{-0.68793, 1.00884, 1.45843}, {-1.89376, 0.108633, 0.141015}, 
      {-0.805874, -0.987141, 1.41214}}, 
     {{-1.89376, 0.108633, 0.141015}, {-0.980538, -1.55178, -0.498532}, 
      {-0.805874, -0.987141, 1.41214}}, 
     {{-0.970542, 0.0952288, -1.6331}, {-0.789696, 1.67778, -0.423627}, 
      {0.805874, 0.987141, -1.41214}}, 
     {{-0.789696, 1.67778, -0.423627}, {-0.68793, 1.00884, 1.45843}, 
      {0.980538, 1.55178, 0.498532}}, 
     {{0.805874, 0.987141, -1.41214}, {-0.789696, 1.67778, -0.423627}, 
      {0.980538, 1.55178, 0.498532}}, 
     {{-0.980538, -1.55178, -0.498532}, {-0.970542, 0.0952288, -1.6331}, 
      {0.68793, -1.00884, -1.45843}}, 
     {{-0.970542, 0.0952288, -1.6331}, {0.805874, 0.987141, -1.41214}, 
      {0.68793, -1.00884, -1.45843}}, 
     {{-0.68793, 1.00884, 1.45843}, {-0.805874, -0.987141, 1.41214}, 
      {0.970542, -0.0952288, 1.6331}}, 
     {{0.980538, 1.55178, 0.498532}, {-0.68793, 1.00884, 1.45843}, 
      {0.970542, -0.0952288, 1.6331}}, 
     {{-0.805874, -0.987141, 1.41214}, {-0.980538, -1.55178, -0.498532}, 
      {0.789696, -1.67778, 0.423627}}, 
     {{-0.980538, -1.55178, -0.498532}, {0.68793, -1.00884, -1.45843}, 
      {0.789696, -1.67778, 0.423627}}, 
     {{0.970542, -0.0952288, 1.6331}, {-0.805874, -0.987141, 1.41214}, 
      {0.789696, -1.67778, 0.423627}}, 
     {{0.805874, 0.987141, -1.41214}, {0.980538, 1.55178, 0.498532}, 
      {1.89376, -0.108633, -0.141015}}, 
     {{0.68793, -1.00884, -1.45843}, {0.805874, 0.987141, -1.41214}, 
      {1.89376, -0.108633, -0.141015}}, 
     {{0.980538, 1.55178, 0.498532}, {0.970542, -0.0952288, 1.6331}, 
      {1.89376, -0.108633, -0.141015}}, 
     {{0.789696, -1.67778, 0.423627}, {0.68793, -1.00884, -1.45843}, 
      {1.89376, -0.108633, -0.141015}}, 
     {{0.970542, -0.0952288, 1.6331}, {0.789696, -1.67778, 0.423627}, 
      {1.89376, -0.108633, -0.141015}}}


`DodecahedronCoords =
     {{{-0.7825577082991626, -0.6228971357138735, -0.3814454420376818}, 
         {-1.056166429126513, -0.1211220152880523, 0.125459017806713}, 
         {-0.8901519729667416, 0.581387025624049, -0.1245657465347393}, 
         {-0.5139406756088238, 0.5137863698859971, -0.7859940087713331}, 
         {-0.4474437630497079, -0.2305021739340014, -0.9447543916118743}}, 
        {{0.2462420398971588, 1.041216149224077, 0.03364256429854498}, 
         {0.1883894489529528, 0.7979763972878896, -0.6882158953736547}, 
         {-0.5139406756088238, 0.5137863698859971, -0.7859940087713331}, 
         {-0.8901519729667416, 0.581387025624049, -0.1245657465347393}, 
         {-0.4203332171238519, 0.9073565559338415, 0.3819975140449359}}, 
        {{0.6889502498141903, 0.2293269496660248, -0.7865460807785904}, 
         {0.2959837136205288, -0.4063077640500321, -0.9450955908765937}, 
         {-0.4474437630497079, -0.2305021739340014, -0.9447543916118743}, 
         {-0.5139406756088238, 0.5137863698859971, -0.7859940087713331}, 
         {0.1883894489529528, 0.7979763972878896, -0.6882158953736547}}, 
        {{0.4203332171238516, -0.9073565559338431, -0.3819975140449322}, 
         {-0.2462420398971585, -1.041216149224077, -0.03364256429854634}, 
         {-0.7825577082991626, -0.6228971357138735, -0.3814454420376818}, 
         {-0.4474437630497079, -0.2305021739340014, -0.9447543916118743}, 
         {0.2959837136205288, -0.4063077640500321, -0.9450955908765937}}, 
        {{-0.1883894489529525, -0.7979763972878881, 0.6882158953736598}, 
         {-0.6889502498141896, -0.2293269496660275, 0.7865460807785892}, 
         {-1.056166429126513, -0.1211220152880523, 0.125459017806713}, 
         {-0.7825577082991626, -0.6228971357138735, -0.3814454420376818}, 
         {-0.2462420398971585, -1.041216149224077, -0.03364256429854634}}, 
        {{-0.2959837136205281, 0.4063077640500364, 0.9450955908765951}, 
         {-0.4203332171238519, 0.9073565559338415, 0.3819975140449359}, 
         {-0.8901519729667416, 0.581387025624049, -0.1245657465347393}, 
         {-1.056166429126513, -0.1211220152880523, 0.125459017806713}, 
         {-0.6889502498141896, -0.2293269496660275, 0.7865460807785892}}, 
        {{-0.6889502498141896, -0.2293269496660275, 0.7865460807785892}, 
         {-0.1883894489529525, -0.7979763972878881, 0.6882158953736598}, 
         {0.513940675608826, -0.5137863698859913, 0.7859940087713382}, 
         {0.4474437630497106, 0.2305021739340082, 0.9447543916118766}, 
         {-0.2959837136205281, 0.4063077640500364, 0.9450955908765951}}, 
        {{-0.2462420398971585, -1.041216149224077, -0.03364256429854634}, 
         {0.4203332171238516, -0.9073565559338431, -0.3819975140449322}, 
         {0.8901519729667409, -0.5813870256240495, 0.1245657465347427}, 
         {0.513940675608826, -0.5137863698859913, 0.7859940087713382}, 
         {-0.1883894489529525, -0.7979763972878881, 0.6882158953736598}}, 
        {{0.2959837136205288, -0.4063077640500321, -0.9450955908765937}, 
         {0.6889502498141903, 0.2293269496660248, -0.7865460807785904}, 
         {1.056166429126513, 0.1211220152880489, -0.1254590178067164}, 
         {0.8901519729667409, -0.5813870256240495, 0.1245657465347427}, 
         {0.4203332171238516, -0.9073565559338431, -0.3819975140449322}}, 
        {{0.1883894489529528, 0.7979763972878896, -0.6882158953736547}, 
         {0.2462420398971588, 1.041216149224077, 0.03364256429854498}, 
         {0.7825577082991649, 0.6228971357138754, 0.3814454420376767}, 
         {1.056166429126513, 0.1211220152880489, -0.1254590178067164}, 
         {0.6889502498141903, 0.2293269496660248, -0.7865460807785904}}, 
        {{-0.4203332171238519, 0.9073565559338415, 0.3819975140449359}, 
         {-0.2959837136205281, 0.4063077640500364, 0.9450955908765951}, 
         {0.4474437630497106, 0.2305021739340082, 0.9447543916118766}, 
         {0.7825577082991649, 0.6228971357138754, 0.3814454420376767}, 
         {0.2462420398971588, 1.041216149224077, 0.03364256429854498}}, 
        {{0.7825577082991649, 0.6228971357138754, 0.3814454420376767}, 
         {0.4474437630497106, 0.2305021739340082, 0.9447543916118766}, 
         {0.513940675608826, -0.5137863698859913, 0.7859940087713382}, 
         {0.8901519729667409, -0.5813870256240495, 0.1245657465347427}, 
         {1.056166429126513, 0.1211220152880489, -0.1254590178067164}}}

`OctahedronCoords =
     {{{0., 0., 1.41421}, {1.41421, 0., 0.}, {0., -1.41421, 0.}}, 
      {{0., 0., 1.41421}, {0., 1.41421, 0.}, {1.41421, 0., 0.}}, 
      {{0., 0., 1.41421}, {-1.41421, 0., 0.}, {0., 1.41421, 0.}}, 
      {{0., 0., 1.41421}, {0., -1.41421, 0.}, {-1.41421, 0., 0.}}, 
      {{0., 0., -1.41421}, {0., -1.41421, 0.}, {1.41421, 0., 0.}}, 
      {{0., 0., -1.41421}, {1.41421, 0., 0.}, {0., 1.41421, 0.}}, 
      {{0., 0., -1.41421}, {0., 1.41421, 0.}, {-1.41421, 0., 0.}}, 
      {{0., 0., -1.41421}, {-1.41421, 0., 0.}, {0., -1.41421, 0.}}}

`CubeCoords =
     {{{0.707107, 0.707107, 0.707107}, {0.707107, 0.707107, -0.707107}, 
         {-0.707107, 0.707107, -0.707107}, {-0.707107, 0.707107, 0.707107}}, 
        {{-0.707107, 0.707107, 0.707107}, {-0.707107, 0.707107, -0.707107}, 
         {-0.707107, -0.707107, -0.707107}, {-0.707107, -0.707107, 0.707107}},
           {{-0.707107, -0.707107, 0.707107}, 
         {-0.707107, -0.707107, -0.707107}, {0.707107, -0.707107, -0.707107}, 
         {0.707107, -0.707107, 0.707107}}, 
        {{0.707107, -0.707107, 0.707107}, {0.707107, -0.707107, -0.707107}, 
         {0.707107, 0.707107, -0.707107}, {0.707107, 0.707107, 0.707107}}, 
        {{0.707107, 0.707107, 0.707107}, {-0.707107, 0.707107, 0.707107}, 
         {-0.707107, -0.707107, 0.707107}, {0.707107, -0.707107, 0.707107}}, 
        {{-0.707107, 0.707107, -0.707107}, {0.707107, 0.707107, -0.707107}, 
         {0.707107, -0.707107, -0.707107}, {-0.707107, -0.707107, -0.707107}}}

`TetrahedronCoords =

    {{{0., 0., 1.}, {-0.942809, 0., -0.333333},
      {0.471405, -0.834546, -0.333333}} ,
     {{0., 0., 1.}, {0.471405, 0.834546, -0.333333}, 
      {-0.942809, 0., -0.333333}}, 
     {{0., 0., 1.}, {0.471405, -0.834546, -0.333333}, 
      {0.471405, 0.834546, -0.333333}}, 
     {{-0.942809, 0., -0.333333}, {0.471405, 0.834546, -0.333333}, 
      {0.471405, -0.834546, -0.333333}}}

EndAdd[]

Null
